# Files_Omapi.pm
# Functions and variables specific to the handling of files and omapi in netreg
# Netreg 1.3rc2-a
#
#----------------------------------------------------------------------------\
# read the leases (file) information into an @array, and return the array
#----------------------------------------------------------------------------\
# 
# all subroutines return undef on error, or a two element array where the 
# first is undef, and the  second is the error string/code/whatever.

package Files_Omapi;
require Exporter;

our @ISA = qw (Exporter);
our @EXPORT = qw (  %LOCS read_leases read_leases_omapi read_conf_new read_conf append_conf write_conf_new append_error_log omapi_init omapi_exec );
our @EXPORT_OK = ( );
our $VERSION = 1.3.2;

use Variables;
use Error;
#use Subnets;

use MIME::Base64;

use Fcntl qw(:DEFAULT :flock); # for file locking


sub read_leases {
# ----------------FILE locking code start ----------------------
    if (!open(LEASES, "<$LEASESPATH/$LEASESFILE")) {
      admin_error ( "Can not open $LEASESPATH/$LEASESFILE for reading: $! <br>\n");
      return undef; 
    }
    unless (flock(LEASES, LOCK_SH | LOCK_NB)) {
      local $| = 1;
      print "Waiting for SHARED lock on $LEASESPATH/$LEASESFILE please be patient...";
      flock(LEASES, LOCK_SH) or die ("Error: can't lock $LEASESPATH/$LEASESFILE file: $!\n");
      print "got lock. continuing.\n" if defined $DEBUG;
    }
# ----------------FILE locking code end ----------------------
  my @return;
  eval {
    local $/  = "}\n";  # make the record separator a curly brace
    @return = <LEASES>;
  };
# ----------------FILE locking code start ----------------------
  close LEASES; # unlocks it too
# ----------------FILE locking code end ----------------------
return @return;
}
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# via an omapi connection handle, get back all the lease info for all 
# hosts.. the goal of this is to make it a drop-in replacement for 
# the non-omapi read_leases
# Prerequisite: a call to parse_subnets()
#----------------------------------------------------------------------------\
sub read_leases_omapi {
my @subnets = @_;
my @tmpsub=@subnets;

my @leases=(); # will be a read_leases compliant array

    foreach my $ServerIP (@DHCP_Servers) {
      my $Cn = omapi_init ($ServerIP, $OMAPIKeyName, $OMAPIKey, $OMAPIPort);
      die (" failed to get connection handle to $ServerIP\n") unless defined $Cn;

      # HACK, what subnet does each dhcp server handle? don't know? , check em all!  
      # this is a Definite performance hit if you have multiple dhcp servers doing
      # different subnets!

      foreach my $subnet ( @tmpsub ) {   # I could use ( keys %LOC ) but it dousn't work
        $subnet =~ s/\d+\/.*$//;
       
      # HACK, should determine range of IP's from Subnets.pm
      my @a = (1..254);
        foreach my $IP (@a) {
          $IP = $subnet.$IP;
          my $LeaseInfo = omapi_exec_leaseinfo($Cn, $IP);
           push @leases, $LeaseInfo if defined $LeaseInfo;
#           print $LeaseInfo if defined $LeaseInfo;
                
        }
      }
    }

  return @leases;
}
#----------------------------------------------------------------------------/



#----------------------------------------------------------------------------\
# read the dhcpd.conf.new (file) information into an @array, and return the array
#----------------------------------------------------------------------------\
sub read_conf {
# ----------------FILE locking code start ----------------------
    if (!open(DHCPDCONF, "<$DHCPDCONFPATH/$DHCPDCONFFILE")) {
      admin_error ( "Can not open $DHCPDCONFPATH/$DHCPDCONFFILE for reading: $! <br>\n");
      return undef;
    }
    unless (flock(DHCPDCONF, LOCK_SH | LOCK_NB)) {
      local $| = 1;
      print "Waiting for SHARED lock on $DHCPDCONFPATH/$DHCPDCONFFILE please be patient...";
      flock(DHCPDCONF, LOCK_SH) or die ("Error: can't lock $DHCPDCONFPATH/$DHCPDCONFFILE file: $!\n");
      print "got lock. continuing.\n" if defined $DEBUG;
    }
# ----------------FILE locking code end ----------------------
my @return = <DHCPDCONF>;
# ----------------FILE locking code start ----------------------
    close DHCPDCONF; # unlocks it too
# ----------------FILE locking code end ----------------------
return @return;
}
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# read the dhcpd.conf.new (file) information into an @array, and return the array
#----------------------------------------------------------------------------\
sub read_conf_new {
# ----------------FILE locking code start ----------------------
    if (!open(DHCPDCONFNEW, "<$DHCPDCONFPATH/$DHCPDCONFFILE.new")) {
      admin_error ( "Can not open $DHCPDCONFPATH/$DHCPDCONFFILE.new for reading: $! <br>\n");
      return undef;
    }
    unless (flock(DHCPDCONFNEW, LOCK_SH | LOCK_NB)) {
      local $| = 1;
      print "Waiting for SHARED lock on $DHCPDCONFPATH/$DHCPDCONFFILE.new please be patient...";
      flock(DHCPDCONFNEW, LOCK_SH) or die ("Error: can't lock $DHCPDCONFPATH/$DHCPDCONFFILE.new file: $!\n");
      print "got lock. continuing.\n" if defined $DEBUG;
    }
# ----------------FILE locking code end ----------------------
my @return = <DHCPDCONFNEW>;
# ----------------FILE locking code start ----------------------
    close DHCPDCONFNEW; # unlocks it too
# ----------------FILE locking code end ----------------------
return @return;
}
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# append to the dhcpd.conf.new (file) information from the parameter @array, and return the new array
#----------------------------------------------------------------------------\
sub append_conf {
my @appendlines = @_;
# ----------------FILE locking code start ----------------------
    if (!sysopen(DHCPDCONFNEW, "$DHCPDCONFPATH/$DHCPDCONFFILE.new", O_WRONLY | O_APPEND | O_CREAT )) {
      admin_error ( "Can not open $DHCPDCONFPATH/$DHCPDCONFFILE.new for appended writing: $! <br>\n");
      return undef;
    }
    unless (flock(DHCPDCONFNEW, LOCK_EX | LOCK_NB)) {
      local $| = 1;
      print "Waiting for EXCLUSIVE lock on $DHCPDCONFPATH/$DHCPDCONFFILE.new please be patient...";
      flock(DHCPDCONFNEW, LOCK_EX) or die ("Error: can't lock $DHCPDCONFPATH/$DHCPDCONFFILE.new file: $!\n");
      print "got lock. continuing.\n" if defined $DEBUG;
    }
# ----------------FILE locking code end ----------------------

    print DHCPDCONFNEW @appendlines;

# ----------------FILE locking code start ----------------------
    close DHCPDCONFNEW; # also unlocks it too
# ----------------FILE locking code end ----------------------
    return 1;
}
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# write out the entire dhcpd.conf.new (file) information from the parameter @array
#----------------------------------------------------------------------------\
sub write_conf_new {
my @confnew = @_;

# ----------------FILE locking code start ----------------------
    if (!sysopen(DHCPDCONFNEW, "$DHCPDCONFPATH/$DHCPDCONFFILE.new", O_WRONLY | O_CREAT )) {
      admin_error ( "Can not open $DHCPDCONFPATH/$DHCPDCONFFILE.new for overwrite/writing: $! <br>\n");
      return undef;
    } 
    unless (flock(DHCPDCONFNEW, LOCK_EX | LOCK_NB)) {
      local $| = 1;
      print "Waiting for EXCLUSIVE lock on $DHCPDCONFPATH/$DHCPDCONFFILE.new please be patient...";
      flock(DHCPDCONFNEW, LOCK_EX) or die ("Error: can't lock $DHCPDCONFPATH/$DHCPDCONFFILE.new file: $!\n");
      print "got lock. continuing.\n" if defined $DEBUG; 
      truncate(DHCPDCONFNEW,0);
    } 
# ----------------FILE locking code end ----------------------

print DHCPDCONFNEW @confnew;

# ----------------FILE locking code start ----------------------
    close DHCPDCONFNEW; # also unlocks it too
# ----------------FILE locking code end ----------------------
     return 1; 
}
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# obviously, this subroute sends the contents of it's input array to the logfile
#----------------------------------------------------------------------------\
sub append_error_log {
my @data = @_;

# ----------------FILE locking code start ----------------------
    if (!sysopen( ERROR_LOG, "$ERROR_LOG", O_WRONLY | O_APPEND | O_CREAT )) {
      print STDERR "Can not open $ERROR_LOG for appended writing: $! <br>\n";
      die ( "Can not open $ERROR_LOG for appended writing: $! <br>\n");
      return undef;
    }
    unless (flock( ERROR_LOG, LOCK_EX | LOCK_NB)) {
      local $| = 1;
      print "Waiting for EXCLUSIVE lock on $ERROR_LOG please be patient...";
      flock( ERROR_LOG, LOCK_EX) or die ("Error: can't lock $ERROR_LOG file: $!\n");
      print "got lock. continuing.\n" if defined $DEBUG;
    }
# ----------------FILE locking code end ----------------------
  
 print ERROR_LOG  @data;

# ----------------FILE locking code start ----------------------
    close ERROR_LOG;
# ----------------FILE locking code end ----------------------
    return 1;
}
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# initiate an omapi connection, and get a connection handle for use later.
#----------------------------------------------------------------------------\
sub omapi_init {
  my ($ServerIP, $KeyName, $Key, $Port) = @_;
  my $BKey = decode_base64($Key);
  my $Conn = eval { new OMAPI::DHCP($ServerIP, $Port, "$KeyName:$BKey"); };
  return undef unless defined $Conn;
  return $Conn; # return connection handle
}
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# via an omapi connection handle, get back any MAC 
# address associated with a leased IP 
#----------------------------------------------------------------------------\
sub omapi_exec_leasemac {
# takes a connection handle, and an IP to look up the MAC for..
  my ($Conn, $IP) = @_;
  my $Lease = $Conn->Select_Lease({'ip-address' => $IP});
  return (undef, ['omapi_lease']) if (!defined $Lease);
  return (undef, ['lease_contents']) if (!defined $Lease->{'hardware-address'});
  my $HA = $Lease->{'hardware-address'};
  return (undef, ['hardware-address']) if ($HA eq '');
  $HA = unpack('H12', $HA);
  $HA =~ s/(..)/$1\0/g;
  $HA = join(":", split(/\0/, $HA));
  return (1, $HA);
}
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# via an omapi connection handle, get back all possible info
# associated with a leased IP 
#----------------------------------------------------------------------------\
sub omapi_exec_leaseinfo {
# takes a connection handle, and an IP to look up the MAC for..
  my ($Conn, $IP) = @_;

  my $Lease = $Conn->Select_Lease({'ip-address' => $IP});
  return (undef, ['omapi_lease']) if (!defined $Lease);

  my %LeaseInfo;
  my $leaseinfo;

  $LeaseInfo{'hardware-address'} = $Lease->{'hardware-address'};
  if (defined $LeaseInfo{'hardware-address'}) {
    $LeaseInfo{'hardware-address'} = unpack('H12', $LeaseInfo{'hardware-address'});
    $LeaseInfo{'hardware-address'} =~ s/(..)/$1\0/g;
    $LeaseInfo{'hardware-address'} = join(":", split(/\0/, $LeaseInfo{'hardware-address'}));
  }

  $LeaseInfo{'dhcp-client-identifier'} = $Lease->{'dhcp-client-identifier'};
  if (defined $LeaseInfo{'dhcp-client-identifier'}) {
    $LeaseInfo{'dhcp-client-identifier'} = unpack('H12', $LeaseInfo{'dhcp-client-identifier'});
    $LeaseInfo{'dhcp-client-identifier'} =~ s/(..)/$1\0/g;
    $LeaseInfo{'dhcp-client-identifier'} = join(":", split(/\0/, $LeaseInfo{'dhcp-client-identifier'}));
  }
   
  $LeaseInfo{'state'} = $Lease->{'state'};
  if (defined  $LeaseInfo{'state'}) {
    $LeaseInfo{'state'} = 'free' if $LeaseInfo{'state'}  eq '1';
    $LeaseInfo{'state'} = 'active' if $LeaseInfo{'state'} eq '2';
  }

  $LeaseInfo{'name'} = $Lease->{'name'} if defined  $Lease->{'name'};
  $LeaseInfo{'hardware-type'} = $Lease->{'hardware-type'} if defined $Lease->{'hardware-type'};
  $LeaseInfo{'client-hostname'} = $Lease->{'client-hostname'} if defined  $Lease->{'client-hostname'};
  $LeaseInfo{'subnet'} = $Lease->{'subnet'} if defined $Lease->{'subnet'};
  $LeaseInfo{'pool'} = $Lease->{'pool'} if defined  $Lease->{'pool'};

  $LeaseInfo{'starts'} = $Lease->{'starts'};
  if (defined  $LeaseInfo{'starts'}) {
   $LeaseInfo{'starts'} = make_time($LeaseInfo{'starts'});
  }

  $LeaseInfo{'ends'} = $Lease->{'ends'};
  if (defined  $LeaseInfo{'ends'} ) {
   $LeaseInfo{'ends'} = make_time($LeaseInfo{'ends'});
  }

  $LeaseInfo{'tstp'} = $Lease->{'tstp'};
  if (defined  $LeaseInfo{'tstp'} ) {
   $LeaseInfo{'tstp'} = make_time($LeaseInfo{'tstp'});
  }

  $LeaseInfo{'tsfp'} = $Lease->{'tsfp'} if defined  $Lease->{'tsfp'};
  $LeaseInfo{'cltt'} = $Lease->{'cltt'} if defined  $Lease->{'cltt'};

   $leaseinfo =	"lease $IP {\n  starts ".$LeaseInfo{'starts'}.";\n". 
		"  ends ".$LeaseInfo{'ends'}.";\n".
		"  tstp ".$LeaseInfo{'tstp'}.";\n".
		"  binding state ".$LeaseInfo{'state'}.";\n";
 $leaseinfo .=	$LeaseInfo{'state'} eq 'active'?"  next binding state free;\n":'';
 $leaseinfo.=   "  hardware ethernet ".$LeaseInfo{'hardware-address'}.";\n".
#		"  uid XXXXXXXXXXXXXXXXXXXXXXX;\n".
		"}\n" if defined $LeaseInfo{'hardware-address'};
  
 # print "$leaseinfo" if defined $LeaseInfo{'hardware-address'};
#  print "-------------------------------------------" if defined $LeaseInfo{'hardware-address'};

 return undef unless defined $leaseinfo;
 return $leaseinfo;

}

sub make_time {
my ($epochtime) = @_;
if (defined  $epochtime ) {
  my ($s,$m,$h,$dy,$mo,$yr,$wd,undef,undef) = gmtime($epochtime);
    $yr+=1900;
    $mo++;
    if ($h<10) { $h="0$h"; }
    if ($m<10) { $m="0$m"; }
    if ($s<10) { $s="0$s"; }
    if ($dy<10) { $dy="0$dy"; }
    if ($mo<9) { $mo="0$mo"; }

   return "$wd $yr/$mo/$dy $h:$m:$s";
   #return "$yr/$mo/$dy $h:$m:$s";
   
} else { return undef; }

}

#----------------------------------------------------------------------------/

1;
