# Subnets.pm
# Functions and variables specific to the handling of subnets in CIDR notation
# for NetReg
# Netreg 1.3rc2-a
#

package Subnets;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw ( parse_subnets bad_subnet ip_in_subnet ip2dec %SUBIPS );
our @EXPORT_OK = qw ( );
our $VERSION = 1.3.2;

# Build hash of all possible subnet masks
my $max = 2**32;
my %masks;
for ($i=0; $i<32; $i++) {
    $masks{32-$i} = $max - 2**$i;
}

use Error;
use Variables;

use Fcntl qw(:DEFAULT :flock); # for file locking


# Parse the subnet.dat file and load data into hashes or arrays
our %SUBIPS;

sub parse_subnets {
# ----------------FILE locking code start ----------------------

  my $SUBNETFILE = "/opt/apache/cgi-bin/subnet.dat";

    if (!open(SUBNETDAT, "<$SUBNETFILE")) {
      admin_error ( "Can not open subnet\.dat for reading: $! <br>\n");
      print_custom_footer();
      exit 1;
    }
    unless (flock(SUBNETDAT, LOCK_SH | LOCK_NB)) {
      local $| = 1;
      print "Waiting for SHARED lock on subnet.dat please be patient...";
      flock(SUBNETDAT, LOCK_SH) or die ("Error: can't lock subnet.dat file: $!\n");
      print "got lock. continuing.\n" if defined $DEBUG;
    }
# ----------------FILE locking code end ----------------------

my %LOCS;
my %URSUBS;

#   open(SUBNETDAT, "<$SUBNETFILE") or admin_error( "Cannot open subnet.dat file: $SUBNETFILE"), print_footer(), exit 0;

    while (<SUBNETDAT>) {

	# If the line doesn't begin with a digit, ignore it
	if (/^\d/) {
	    chomp;
	    $_ .= ':';
	    ($SUBNET, $LOC, $SUBIPS, $URSUB) = split /:/;
	    my $test = bad_subnet( $SUBNET );
	    if ($test) {
		admin_error( $test, "Subnet Config" );
	    } else {
		if ($URSUB eq '') {
		    $URSUB = $SUBNET;
		} else {
		    $test = bad_subnet( $URSUB );
		    if ($test) {
			admin_error( $test, "Subnet Config" );
		    }
		}
 		#print "parse_: ",join ('#',$LOC, $SUBIPS, $URSUB),"\n";
		$LOCS{$SUBNET} = join ('#',$LOC, $SUBIPS, $URSUB);
	    }
	}
    }
    close (SUBNETDAT);
    # we now return a complex HASH of info from the subnets.dat file
    return %LOCS;
      
}

# Test if subnet/mask pair is valid
#   Return null if fine, other text of error with corrected subnet
#   address for the given mask.

sub bad_subnet {
    my ($subnet,$mask) = split('/', $_[0]);
    my $psn = ip2dec($subnet);
    my $retval;
    my $real_sn;
    my @rsn;
    my $i;

    if (($psn & $masks{$mask}) != $psn) {
	$retval = "Bad subnet $subnet for provided mask /$mask: should be ";
	$real_sn = ($psn & $masks{$mask});
	for ($i=3; $i>=0; $i--) {
	    $rsn[$i] = $real_sn & 255;
	    $real_sn = $real_sn >> 8;
	}
	$retval .= (join('.', @rsn) . "/$mask");
    }
    return $retval;
}

# Return true or false depending on whether an IP address is in the subnet provided

sub ip_in_subnet {
    my $ip = $_[0];
    #print "IP: $ip SUBNET: $_[1]\n";
    my ($subnet,$mask) = split('/', $_[1]);
    my $pip = ip2dec($ip);
    my $psn = ip2dec($subnet);

    if ( ($pip & $masks{$mask}) == ($psn & $masks{$mask}) ) {
	return 1;    # IP address is in that subnet!
    } else {
	return 0;
    }
}


# Convert dotted-quad IP address to 32-bit int
sub ip2dec {
    ($o1,$o2,$o3,$o4) = split(/\./, $_[0]);
    ($o1 << 24) + ($o2 << 16) + ($o3 << 8) + $o4;
}

# For debugging only -- convert decimal to binary
sub dec2bin {
    return unpack("B32", pack("N", shift));
}

# For debugging only -- convert binary to decimal
sub bin2dec {
    return unpack("N", pack("B32", substr("0" x 32 . shift, -32)));
}

1;
