#!/usr/bin/perl -w
#
#----------------------------------------------------------------------------\
# NetReg 1.3rc2-a
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
#
# register.cgi -- Student Registration Frontend
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Peter Valian <valianp@southwestern.edu>
# http://www.netreg.org/
#
# Code contributions by:
#
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#
#----------------------------------------------------------------------------/

use Variables;
use Subnets;
use Error; 
use Files_Omapi;


#----------------------------------------------------------------------------\
# Main Function
#----------------------------------------------------------------------------\
  my @subnets = parse_subnets();
  get_input();
  check_input();
  authenticate();
  get_host_info();
  append_host_entry();
  print_success();
#----------------------------------------------------------------------------/


#############################################################################\
## SUBROUTINES BEGIN
#############################################################################/


#----------------------------------------------------------------------------\
# Get the input from the CGI FORM
#----------------------------------------------------------------------------\
  sub get_input {
    my $buffer;
    return unless defined $ENV{'CONTENT_LENGTH'};
    read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    @pairs = split(/&/, $buffer);
    foreach $pair (@pairs) {
      ($name, $value) = split(/=/, $pair);
      $value =~ tr/+/ /;
      $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
      $value =~ s/\s+//g;
      $FORM{$name} = $value;
    }
  }
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# Make sure the user is allowed to register and
# we have all the input we need from the user
#----------------------------------------------------------------------------\
  sub check_input {
     my $subnet = '';;
    my $found = 0;
    foreach $subnet (sort keys @subnets) {
      my $unsub = $subnet[2];   # index to the UNregistered subnet associated to a main subnet
      if (ip_in_subnet($IP,$subnet)) {
          $found = 1;
          last;
      }
    }

    if (!$found) {
       register_error("Permission Denied<P>You are not in my allowable IP range<br>$IP<br>$subnet");
    }

    if ((!$FORM{'user'}) || (!$FORM{'pass'})) {
      register_error("Invalid Login");
    }
  }
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# Use the POP or FTP server for user authentication
# POPStat will return the number of messages in user's INBOX or 
# will return -1 if the connection failed, $ftp->pwd is return null if the
# connection failed.
#----------------------------------------------------------------------------\
  sub authenticate {
    # First check to make sure that user and password are not blank.
    if ($FORM{'user'} eq "" || $FORM{'pass'} eq "") { register_error("Username or Password cannot be blank."); }

    if ($AUTH_METHOD eq "POP") {
      $pop=new Mail::POP3Client("$FORM{'user'}","$FORM{'pass'}","$POPSERVER");
      $alive=$pop->POPStat;
      $popmsg=$pop->Message;
      $pop->Close;
      unless ($alive ne "-1") {
        register_error("Authentication Failed<P>POP replied:<br>$popmsg");
      }
    } elsif ($AUTH_METHOD eq "FTP") {
      $ftp = Net::FTP->new($FTPSERVER);
      $ftp->login($FORM{'user'}, $FORM{'pass'});
      $ftp->pwd()||register_error("Authentication Failed<P>Bad Username or Password.");
    } elsif ($AUTH_METHOD eq "IMAP") {
      $imap = new Net::IMAP::Simple( "$IMAPSERVER" );
      $alive = $imap->login( $FORM{'user'}, $FORM{'pass'} );
      $imap->quit();
      unless ($alive) {
        register_error("Authentication Failed");
      }
    } elsif ($AUTH_METHOD eq "LDAP") {
      $ldap = Net::LDAP->new($LDAPSERVER) or die "$@";
      # Do an anonymous bind and search for the dn.
      $error = $ldap->bind;
      if ($error->code) {
        $ldapmsg = ldap_error_desc($error->code);
        register_error("Authentication Failed<P>LDAP replied:<br>$ldapmsg");
      }

      $mesg = $ldap->search (
                             base   => "$LDAP_BASE",
                             filter => "cn=$FORM{'user'}"
                            );
      if ($mesg->code) {
        $ldapmsg = ldap_error_name($mesg->code);
        register_error("Authentication Failed<P>LDAP replied:<br>$ldapmsg");
      }
      $ldap->unbind;
      unless ($mesg->count==1) {
        register_error("The server returned more than one, or zero, matches for $FORM{'user'}.")
      }

      # Now pull the dn out.
      $entry = $mesg->pop_entry;
      $dn = $entry->dn;
      # Now bind again with the dn and password.  No error code means success. 

      $ldap = Net::LDAP->new($LDAPSERVER) or die "$@";
      $error = $ldap->bind (
                            dn       => $dn,
                            password => $FORM{'pass'}
                           );
      if ($error->code) {
        $ldapmsg = ldap_error_name($error->code);
        register_error("Authentication Failed<P>LDAP replied:<br>$ldapmsg");
      }

      $ldap->unbind;
    }
    else {
      register_error("No Authentication Method");
    }
  }
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# The User has authenticated, get their network & hardware info
#----------------------------------------------------------------------------\

sub get_host_info {


    foreach my $ServerIP (@DHCP_Servers) {
      my $Cn = omapi_init ($ServerIP, $OMAPIKeyName, $OMAPIKey, $OMAPIPort);
      die (" failed to get connection handle to $ServerIP\n") unless defined $Cn;
      my ($oRes, $MAC) = omapi_exec_leasemac($Cn, $IP);
      $RECORDS{$IP} = uc($MAC) if defined $oRes;
    } 
    if (!exists ($RECORDS{$IP})) {
       register_error("No Lease Found for your IP of: $IP \nDEBUG:(via OMAPI)");
    }
}


#----------------------------------------------------------------------------\
# Put the date in a usable/sortable format (Y2K Compliant)
# Open dhcpd.conf.new in append mode and add the user as a known host
#----------------------------------------------------------------------------\
  sub append_host_entry {
    # Get the time and put it in the format we want
    ($SEC,$MIN,$HOUR,$DAY,$MO,$YR) = localtime;
    $YR+=1900; $MO++;
    if ($DAY<10) { $DAY="0$DAY"; }
    if ($MO<10) { $MO="0$MO"; }
    if ($HOUR<10) { $HOUR="0$HOUR"; }
    if ($MIN<10) { $MIN="0$MIN"; }
    if ($SEC<10) { $SEC="0$SEC"; }
    $TIME="$HOUR:$MIN:$SEC";

    @RECORDS = (keys %RECORDS);
    # Check if the MAC is already registered

    my @conf = read_conf_new();
    print_custom_footer() unless @conf;
    foreach (@conf) {
      if ($_ =~ /$RECORDS{$IP}/) {
        if ($RECORDS{$IP} eq "") { register_error("No Registration Required"); }
        else {
          register_error("Hardware Already Registered<br>Please Reboot Your Computer <br> ...it may take up to 60 seconds to take effect so take your time ;-)");
        }
      }
    }
 
    @conf = read_conf_new();
    print_custom_footer() unless @conf;
    foreach (@conf) {
      if ($_ =~ /$FORM{'user'}/) {
        ($TRASH, $HOST) = split /\s/;
        push (@HOSTS, $HOST);
      }
    }
    
    my @SHOSTS = (sort { $a <=> $b } @HOSTS);
    ($TRASH, $SERIAL) = split(/-/,$SHOSTS[$#SHOSTS]);
    $SERIAL++;
    
    
    # Add DHCP option to use derived name in DDNS updates, if desired
    my @append;
    if (!$DDNS_USE_HOST_DECL) {
     # NOT DDNS
      @append = (
         "host $FORM{'user'}-$SERIAL { hardware ethernet ".
        "$RECORDS{$IP} \; }\#$ENV{'HTTP_USER_AGENT'}".
        "\#$YR$MO$DAY $TIME\#$IP#$FORM{'orgunit'}#$FORM{'asset'}\n"
      );
    } else {
     # DDNS
      @append = (
         "host $FORM{'user'}-$SERIAL { hardware ethernet ".
        "$RECORDS{$IP} \; ddns-hostname $FORM{'user'}-$SERIAL; }\#$ENV{'HTTP_USER_AGENT'}".
        "\#$YR$MO$DAY $TIME\#$IP#$FORM{'orgunit'}#$FORM{'asset'}\n"
      );
    }

     my $err = append_conf(@append);
     print_custom_footer() unless defined $err;


  }
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# The user has been added, print out success message
#----------------------------------------------------------------------------\
  sub print_success {
    print_custom_header();
    print "Registration Complete.<br>Please wait 60 seconds and then<br>reboot your computer</b>.<P>";
    print_custom_footer();
  }
#----------------------------------------------------------------------------/


